@@ -13,12 +13,14 @@ conda uninstall -y --force \
     rasterio \
     packaging \
     pint \
+    bottleneck \
     sparse \
+    flox \
     h5netcdf \
     xarray
-    # bottleneck \ # re-enable again, see https://github.com/pydata/bottleneck/pull/378
+# new matplotlib dependency
+python -m pip install --upgrade contourpy
 # to limit the runtime of Upstream CI
-python -m pip install pytest-timeout
 python -m pip install \
     -i https://pypi.anaconda.org/scipy-wheels-nightly/simple \
     --no-deps \
@@ -26,13 +28,8 @@ python -m pip install \
     --upgrade \
     numpy \
     scipy \
+    matplotlib \
     pandas
-python -m pip install \
-    -f https://7933911d6844c6c53a7d-47bd50c35cd79bd838daf386af554a83.ssl.cf2.rackcdn.com \
-    --no-deps \
-    --pre \
-    --upgrade \
-    matplotlib
 python -m pip install \
     --no-deps \
     --upgrade \
@@ -40,11 +37,13 @@ python -m pip install \
     git+https://github.com/dask/distributed \
     git+https://github.com/zarr-developers/zarr \
     git+https://github.com/Unidata/cftime \
-    git+https://github.com/mapbox/rasterio \
+    git+https://github.com/rasterio/rasterio \
     git+https://github.com/pypa/packaging \
     git+https://github.com/hgrecco/pint \
+    git+https://github.com/pydata/bottleneck \
     git+https://github.com/pydata/sparse \
     git+https://github.com/intake/filesystem_spec \
     git+https://github.com/SciTools/nc-time-axis \
+    git+https://github.com/xarray-contrib/flox \
     git+https://github.com/h5netcdf/h5netcdf
-    # git+https://github.com/pydata/bottleneck \ # re-enable again, see https://github.com/pydata/bottleneck/pull/378
+python -m pip install pytest-timeout
