@@ -1,6 +1,8 @@
+from __future__ import annotations
+
 import threading
 from collections import OrderedDict
-from typing import Any, Callable, Iterator, MutableMapping, Optional, TypeVar
+from typing import Any, Callable, Iterator, MutableMapping, TypeVar
 
 K = TypeVar("K")
 V = TypeVar("V")
@@ -21,10 +23,10 @@ class LRUCache(MutableMapping[K, V]):
     the cache, e.g., ``cache.maxsize = new_size``.
     """
 
-    _cache: "OrderedDict[K, V]"
+    _cache: OrderedDict[K, V]
     _maxsize: int
     _lock: threading.RLock
-    _on_evict: Optional[Callable[[K, V], Any]]
+    _on_evict: Callable[[K, V], Any] | None
 
     __slots__ = ("_cache", "_lock", "_maxsize", "_on_evict")
 
@@ -34,7 +36,7 @@ class LRUCache(MutableMapping[K, V]):
         ----------
         maxsize : int
             Integer maximum number of items to hold in the cache.
-        on_evict: callable, optional
+        on_evict : callable, optional
             Function to call like ``on_evict(key, value)`` when items are
             evicted.
         """
@@ -55,8 +57,7 @@ class LRUCache(MutableMapping[K, V]):
             return value
 
     def _enforce_size_limit(self, capacity: int) -> None:
-        """Shrink the cache if necessary, evicting the oldest items.
-        """
+        """Shrink the cache if necessary, evicting the oldest items."""
         while len(self._cache) > capacity:
             key, value = self._cache.popitem(last=False)
             if self._on_evict is not None:
