@@ -1,23 +1,44 @@
+from __future__ import annotations
+
 import warnings
-from distutils.version import LooseVersion
+from typing import TYPE_CHECKING, Generic
 
 import numpy as np
 import pandas as pd
 
+from ..coding.times import infer_calendar_name
 from .common import (
     _contains_datetime_like_objects,
     is_np_datetime_like,
     is_np_timedelta_like,
 )
+from .npcompat import DTypeLike
 from .pycompat import is_duck_dask_array
+from .types import T_DataArray
+
+if TYPE_CHECKING:
+    from .dataarray import DataArray
+    from .dataset import Dataset
+    from .types import CFCalendar
 
 
 def _season_from_months(months):
     """Compute season (DJF, MAM, JJA, SON) from month ordinal"""
     # TODO: Move "season" accessor upstream into pandas
-    seasons = np.array(["DJF", "MAM", "JJA", "SON"])
+    seasons = np.array(["DJF", "MAM", "JJA", "SON", "nan"])
     months = np.asarray(months)
-    return seasons[(months // 3) % 4]
+
+    with warnings.catch_warnings():
+        warnings.filterwarnings(
+            "ignore", message="invalid value encountered in floor_divide"
+        )
+        warnings.filterwarnings(
+            "ignore", message="invalid value encountered in remainder"
+        )
+        idx = (months // 3) % 4
+
+    idx[np.isnan(idx)] = 4
+    return seasons[idx.astype(int)]
 
 
 def _access_through_cftimeindex(values, name):
@@ -30,6 +51,10 @@ def _access_through_cftimeindex(values, name):
     if name == "season":
         months = values_as_cftimeindex.month
         field_values = _season_from_months(months)
+    elif name == "date":
+        raise AttributeError(
+            "'CFTimeIndex' object has no attribute `date`. Consider using the floor method instead, for instance: `.time.dt.floor('D')`."
+        )
     else:
         field_values = getattr(values_as_cftimeindex, name)
     return field_values.reshape(values.shape)
@@ -140,7 +165,7 @@ def _round_field(values, name, freq):
         return _round_through_series_or_index(values, name, freq)
 
 
-def _strftime_through_cftimeindex(values, date_format):
+def _strftime_through_cftimeindex(values, date_format: str):
     """Coerce an array of cftime-like values to a CFTimeIndex
     and access requested datetime component
     """
@@ -152,7 +177,7 @@ def _strftime_through_cftimeindex(values, date_format):
     return field_values.values.reshape(values.shape)
 
 
-def _strftime_through_series(values, date_format):
+def _strftime_through_series(values, date_format: str):
     """Coerce an array of datetime-like values to a pandas Series and
     apply string formatting
     """
@@ -174,32 +199,26 @@ def _strftime(values, date_format):
         return access_method(values, date_format)
 
 
-class Properties:
-    def __init__(self, obj):
+class TimeAccessor(Generic[T_DataArray]):
+
+    __slots__ = ("_obj",)
+
+    def __init__(self, obj: T_DataArray) -> None:
         self._obj = obj
 
-    def _tslib_field_accessor(  # type: ignore
-        name: str, docstring: str = None, dtype: np.dtype = None
-    ):
-        def f(self, dtype=dtype):
-            if dtype is None:
-                dtype = self._obj.dtype
-            obj_type = type(self._obj)
-            result = _get_date_field(self._obj.data, name, dtype)
-            return obj_type(
-                result, name=name, coords=self._obj.coords, dims=self._obj.dims
-            )
+    def _date_field(self, name: str, dtype: DTypeLike) -> T_DataArray:
+        if dtype is None:
+            dtype = self._obj.dtype
+        obj_type = type(self._obj)
+        result = _get_date_field(self._obj.data, name, dtype)
+        return obj_type(result, name=name, coords=self._obj.coords, dims=self._obj.dims)
 
-        f.__name__ = name
-        f.__doc__ = docstring
-        return property(f)
-
-    def _tslib_round_accessor(self, name, freq):
+    def _tslib_round_accessor(self, name: str, freq: str) -> T_DataArray:
         obj_type = type(self._obj)
         result = _round_field(self._obj.data, name, freq)
         return obj_type(result, name=name, coords=self._obj.coords, dims=self._obj.dims)
 
-    def floor(self, freq):
+    def floor(self, freq: str) -> T_DataArray:
         """
         Round timestamps downward to specified frequency resolution.
 
@@ -216,7 +235,7 @@ class Properties:
 
         return self._tslib_round_accessor("floor", freq)
 
-    def ceil(self, freq):
+    def ceil(self, freq: str) -> T_DataArray:
         """
         Round timestamps upward to specified frequency resolution.
 
@@ -232,7 +251,7 @@ class Properties:
         """
         return self._tslib_round_accessor("ceil", freq)
 
-    def round(self, freq):
+    def round(self, freq: str) -> T_DataArray:
         """
         Round timestamps to specified frequency resolution.
 
@@ -249,7 +268,7 @@ class Properties:
         return self._tslib_round_accessor("round", freq)
 
 
-class DatetimeAccessor(Properties):
+class DatetimeAccessor(TimeAccessor[T_DataArray]):
     """Access datetime fields for DataArrays with datetime-like dtypes.
 
     Fields can be accessed through the `.dt` attribute
@@ -257,8 +276,6 @@ class DatetimeAccessor(Properties):
 
     Examples
     ---------
-    >>> import xarray as xr
-    >>> import pandas as pd
     >>> dates = pd.date_range(start="2000/01/01", freq="D", periods=10)
     >>> ts = xr.DataArray(dates, dims=("time"))
     >>> ts
@@ -286,7 +303,7 @@ class DatetimeAccessor(Properties):
 
     """
 
-    def strftime(self, date_format):
+    def strftime(self, date_format: str) -> T_DataArray:
         """
         Return an array of formatted strings specified by date_format, which
         supports the same string format as the python standard library. Details
@@ -319,11 +336,11 @@ class DatetimeAccessor(Properties):
             result, name="strftime", coords=self._obj.coords, dims=self._obj.dims
         )
 
-    def isocalendar(self):
+    def isocalendar(self) -> Dataset:
         """Dataset containing ISO year, week number, and weekday.
 
-        Note
-        ----
+        Notes
+        -----
         The iso year and weekday differ from the nominal year and weekday.
         """
 
@@ -332,9 +349,6 @@ class DatetimeAccessor(Properties):
         if not is_np_datetime_like(self._obj.data.dtype):
             raise AttributeError("'CFTimeIndex' object has no attribute 'isocalendar'")
 
-        if LooseVersion(pd.__version__) < "1.1.0":
-            raise AttributeError("'isocalendar' not available in pandas < 1.1.0")
-
         values = _get_date_field(self._obj.data, "isocalendar", np.int64)
 
         obj_type = type(self._obj)
@@ -346,31 +360,48 @@ class DatetimeAccessor(Properties):
 
         return Dataset(data_vars)
 
-    year = Properties._tslib_field_accessor(
-        "year", "The year of the datetime", np.int64
-    )
-    month = Properties._tslib_field_accessor(
-        "month", "The month as January=1, December=12", np.int64
-    )
-    day = Properties._tslib_field_accessor("day", "The days of the datetime", np.int64)
-    hour = Properties._tslib_field_accessor(
-        "hour", "The hours of the datetime", np.int64
-    )
-    minute = Properties._tslib_field_accessor(
-        "minute", "The minutes of the datetime", np.int64
-    )
-    second = Properties._tslib_field_accessor(
-        "second", "The seconds of the datetime", np.int64
-    )
-    microsecond = Properties._tslib_field_accessor(
-        "microsecond", "The microseconds of the datetime", np.int64
-    )
-    nanosecond = Properties._tslib_field_accessor(
-        "nanosecond", "The nanoseconds of the datetime", np.int64
-    )
+    @property
+    def year(self) -> T_DataArray:
+        """The year of the datetime"""
+        return self._date_field("year", np.int64)
 
     @property
-    def weekofyear(self):
+    def month(self) -> T_DataArray:
+        """The month as January=1, December=12"""
+        return self._date_field("month", np.int64)
+
+    @property
+    def day(self) -> T_DataArray:
+        """The days of the datetime"""
+        return self._date_field("day", np.int64)
+
+    @property
+    def hour(self) -> T_DataArray:
+        """The hours of the datetime"""
+        return self._date_field("hour", np.int64)
+
+    @property
+    def minute(self) -> T_DataArray:
+        """The minutes of the datetime"""
+        return self._date_field("minute", np.int64)
+
+    @property
+    def second(self) -> T_DataArray:
+        """The seconds of the datetime"""
+        return self._date_field("second", np.int64)
+
+    @property
+    def microsecond(self) -> T_DataArray:
+        """The microseconds of the datetime"""
+        return self._date_field("microsecond", np.int64)
+
+    @property
+    def nanosecond(self) -> T_DataArray:
+        """The nanoseconds of the datetime"""
+        return self._date_field("nanosecond", np.int64)
+
+    @property
+    def weekofyear(self) -> DataArray:
         "The week ordinal of the year"
 
         warnings.warn(
@@ -379,78 +410,108 @@ class DatetimeAccessor(Properties):
             FutureWarning,
         )
 
-        if LooseVersion(pd.__version__) < "1.1.0":
-            weekofyear = Properties._tslib_field_accessor(
-                "weekofyear", "The week ordinal of the year", np.int64
-            ).fget(self)
-        else:
-            weekofyear = self.isocalendar().week
+        weekofyear = self.isocalendar().week
 
         return weekofyear
 
     week = weekofyear
-    dayofweek = Properties._tslib_field_accessor(
-        "dayofweek", "The day of the week with Monday=0, Sunday=6", np.int64
-    )
+
+    @property
+    def dayofweek(self) -> T_DataArray:
+        """The day of the week with Monday=0, Sunday=6"""
+        return self._date_field("dayofweek", np.int64)
+
     weekday = dayofweek
 
-    weekday_name = Properties._tslib_field_accessor(
-        "weekday_name", "The name of day in a week", object
-    )
+    @property
+    def weekday_name(self) -> T_DataArray:
+        """The name of day in a week"""
+        return self._date_field("weekday_name", object)
+
+    @property
+    def dayofyear(self) -> T_DataArray:
+        """The ordinal day of the year"""
+        return self._date_field("dayofyear", np.int64)
+
+    @property
+    def quarter(self) -> T_DataArray:
+        """The quarter of the date"""
+        return self._date_field("quarter", np.int64)
+
+    @property
+    def days_in_month(self) -> T_DataArray:
+        """The number of days in the month"""
+        return self._date_field("days_in_month", np.int64)
 
-    dayofyear = Properties._tslib_field_accessor(
-        "dayofyear", "The ordinal day of the year", np.int64
-    )
-    quarter = Properties._tslib_field_accessor("quarter", "The quarter of the date")
-    days_in_month = Properties._tslib_field_accessor(
-        "days_in_month", "The number of days in the month", np.int64
-    )
     daysinmonth = days_in_month
 
-    season = Properties._tslib_field_accessor("season", "Season of the year", object)
+    @property
+    def season(self) -> T_DataArray:
+        """Season of the year"""
+        return self._date_field("season", object)
 
-    time = Properties._tslib_field_accessor(
-        "time", "Timestamps corresponding to datetimes", object
-    )
+    @property
+    def time(self) -> T_DataArray:
+        """Timestamps corresponding to datetimes"""
+        return self._date_field("time", object)
 
-    is_month_start = Properties._tslib_field_accessor(
-        "is_month_start",
-        "Indicates whether the date is the first day of the month.",
-        bool,
-    )
-    is_month_end = Properties._tslib_field_accessor(
-        "is_month_end", "Indicates whether the date is the last day of the month.", bool
-    )
-    is_quarter_start = Properties._tslib_field_accessor(
-        "is_quarter_start",
-        "Indicator for whether the date is the first day of a quarter.",
-        bool,
-    )
-    is_quarter_end = Properties._tslib_field_accessor(
-        "is_quarter_end",
-        "Indicator for whether the date is the last day of a quarter.",
-        bool,
-    )
-    is_year_start = Properties._tslib_field_accessor(
-        "is_year_start", "Indicate whether the date is the first day of a year.", bool
-    )
-    is_year_end = Properties._tslib_field_accessor(
-        "is_year_end", "Indicate whether the date is the last day of the year.", bool
-    )
-    is_leap_year = Properties._tslib_field_accessor(
-        "is_leap_year", "Boolean indicator if the date belongs to a leap year.", bool
-    )
+    @property
+    def date(self) -> T_DataArray:
+        """Date corresponding to datetimes"""
+        return self._date_field("date", object)
+
+    @property
+    def is_month_start(self) -> T_DataArray:
+        """Indicate whether the date is the first day of the month"""
+        return self._date_field("is_month_start", bool)
+
+    @property
+    def is_month_end(self) -> T_DataArray:
+        """Indicate whether the date is the last day of the month"""
+        return self._date_field("is_month_end", bool)
+
+    @property
+    def is_quarter_start(self) -> T_DataArray:
+        """Indicate whether the date is the first day of a quarter"""
+        return self._date_field("is_quarter_start", bool)
+
+    @property
+    def is_quarter_end(self) -> T_DataArray:
+        """Indicate whether the date is the last day of a quarter"""
+        return self._date_field("is_quarter_end", bool)
+
+    @property
+    def is_year_start(self) -> T_DataArray:
+        """Indicate whether the date is the first day of a year"""
+        return self._date_field("is_year_start", bool)
+
+    @property
+    def is_year_end(self) -> T_DataArray:
+        """Indicate whether the date is the last day of the year"""
+        return self._date_field("is_year_end", bool)
+
+    @property
+    def is_leap_year(self) -> T_DataArray:
+        """Indicate if the date belongs to a leap year"""
+        return self._date_field("is_leap_year", bool)
+
+    @property
+    def calendar(self) -> CFCalendar:
+        """The name of the calendar of the dates.
+
+        Only relevant for arrays of :py:class:`cftime.datetime` objects,
+        returns "proleptic_gregorian" for arrays of :py:class:`numpy.datetime64` values.
+        """
+        return infer_calendar_name(self._obj.data)
 
 
-class TimedeltaAccessor(Properties):
+class TimedeltaAccessor(TimeAccessor[T_DataArray]):
     """Access Timedelta fields for DataArrays with Timedelta-like dtypes.
 
     Fields can be accessed through the `.dt` attribute for applicable DataArrays.
 
     Examples
     --------
-    >>> import pandas as pd
-    >>> import xarray as xr
     >>> dates = pd.timedelta_range(start="1 day", freq="6H", periods=20)
     >>> ts = xr.DataArray(dates, dims=("time"))
     >>> ts
@@ -484,28 +545,31 @@ class TimedeltaAccessor(Properties):
       * time     (time) timedelta64[ns] 1 days 00:00:00 ... 5 days 18:00:00
     """
 
-    days = Properties._tslib_field_accessor(
-        "days", "Number of days for each element.", np.int64
-    )
-    seconds = Properties._tslib_field_accessor(
-        "seconds",
-        "Number of seconds (>= 0 and less than 1 day) for each element.",
-        np.int64,
-    )
-    microseconds = Properties._tslib_field_accessor(
-        "microseconds",
-        "Number of microseconds (>= 0 and less than 1 second) for each element.",
-        np.int64,
-    )
-    nanoseconds = Properties._tslib_field_accessor(
-        "nanoseconds",
-        "Number of nanoseconds (>= 0 and less than 1 microsecond) for each element.",
-        np.int64,
-    )
+    @property
+    def days(self) -> T_DataArray:
+        """Number of days for each element"""
+        return self._date_field("days", np.int64)
+
+    @property
+    def seconds(self) -> T_DataArray:
+        """Number of seconds (>= 0 and less than 1 day) for each element"""
+        return self._date_field("seconds", np.int64)
+
+    @property
+    def microseconds(self) -> T_DataArray:
+        """Number of microseconds (>= 0 and less than 1 second) for each element"""
+        return self._date_field("microseconds", np.int64)
+
+    @property
+    def nanoseconds(self) -> T_DataArray:
+        """Number of nanoseconds (>= 0 and less than 1 microsecond) for each element"""
+        return self._date_field("nanoseconds", np.int64)
 
 
-class CombinedDatetimelikeAccessor(DatetimeAccessor, TimedeltaAccessor):
-    def __new__(cls, obj):
+class CombinedDatetimelikeAccessor(
+    DatetimeAccessor[T_DataArray], TimedeltaAccessor[T_DataArray]
+):
+    def __new__(cls, obj: T_DataArray) -> CombinedDatetimelikeAccessor:
         # CombinedDatetimelikeAccessor isn't really instatiated. Instead
         # we need to choose which parent (datetime or timedelta) is
         # appropriate. Since we're checking the dtypes anyway, we'll just
@@ -519,6 +583,6 @@ class CombinedDatetimelikeAccessor(DatetimeAccessor, TimedeltaAccessor):
             )
 
         if is_np_timedelta_like(obj.dtype):
-            return TimedeltaAccessor(obj)
+            return TimedeltaAccessor(obj)  # type: ignore[return-value]
         else:
-            return DatetimeAccessor(obj)
+            return DatetimeAccessor(obj)  # type: ignore[return-value]
