@@ -1,10 +1,9 @@
-import pickle
+from __future__ import annotations
 
 import numpy as np
 import pytest
 
 import xarray as xr
-import xarray.ufuncs as xu
 
 from . import assert_array_equal
 from . import assert_identical as assert_identical_
@@ -158,53 +157,3 @@ def test_gufuncs():
     fake_gufunc = mock.Mock(signature="(n)->()", autospec=np.sin)
     with pytest.raises(NotImplementedError, match=r"generalized ufuncs"):
         xarray_obj.__array_ufunc__(fake_gufunc, "__call__", xarray_obj)
-
-
-def test_xarray_ufuncs_deprecation():
-    with pytest.warns(FutureWarning, match="xarray.ufuncs"):
-        xu.cos(xr.DataArray([0, 1]))
-
-    with pytest.warns(None) as record:
-        xu.angle(xr.DataArray([0, 1]))
-    assert len(record) == 0
-
-
-@pytest.mark.filterwarnings("ignore::RuntimeWarning")
-@pytest.mark.parametrize(
-    "name",
-    [
-        name
-        for name in dir(xu)
-        if (
-            not name.startswith("_")
-            and hasattr(np, name)
-            and name not in ["print_function", "absolute_import", "division"]
-        )
-    ],
-)
-def test_numpy_ufuncs(name, request):
-    x = xr.DataArray([1, 1])
-
-    np_func = getattr(np, name)
-    if hasattr(np_func, "nin") and np_func.nin == 2:
-        args = (x, x)
-    else:
-        args = (x,)
-
-    y = np_func(*args)
-
-    if name in ["angle", "iscomplex"]:
-        # these functions need to be handled with __array_function__ protocol
-        assert isinstance(y, np.ndarray)
-    elif name in ["frexp"]:
-        # np.frexp returns a tuple
-        assert not isinstance(y, xr.DataArray)
-    else:
-        assert isinstance(y, xr.DataArray)
-
-
-@pytest.mark.filterwarnings("ignore:xarray.ufuncs")
-def test_xarray_ufuncs_pickle():
-    a = 1.0
-    cos_pickled = pickle.loads(pickle.dumps(xu.cos))
-    assert_identical(cos_pickled(a), xu.cos(a))
