@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from datetime import datetime
 from typing import Hashable
 
@@ -89,31 +91,6 @@ def test_safe_cast_to_index_datetime_datetime():
     assert isinstance(actual, pd.Index)
 
 
-def test_multiindex_from_product_levels():
-    result = utils.multiindex_from_product_levels(
-        [pd.Index(["b", "a"]), pd.Index([1, 3, 2])]
-    )
-    np.testing.assert_array_equal(
-        result.codes, [[0, 0, 0, 1, 1, 1], [0, 1, 2, 0, 1, 2]]
-    )
-    np.testing.assert_array_equal(result.levels[0], ["b", "a"])
-    np.testing.assert_array_equal(result.levels[1], [1, 3, 2])
-
-    other = pd.MultiIndex.from_product([["b", "a"], [1, 3, 2]])
-    np.testing.assert_array_equal(result.values, other.values)
-
-
-def test_multiindex_from_product_levels_non_unique():
-    result = utils.multiindex_from_product_levels(
-        [pd.Index(["b", "a"]), pd.Index([1, 1, 2])]
-    )
-    np.testing.assert_array_equal(
-        result.codes, [[0, 0, 0, 1, 1, 1], [0, 0, 1, 0, 0, 1]]
-    )
-    np.testing.assert_array_equal(result.levels[0], ["b", "a"])
-    np.testing.assert_array_equal(result.levels[1], [1, 2])
-
-
 class TestArrayEquiv:
     def test_0d(self):
         # verify our work around for pd.isnull not working for 0-dimensional
