@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import numpy as np
 import pytest
 
@@ -194,6 +196,160 @@ def test_weighted_mean_no_nan(weights, expected):
     assert_equal(expected, result)
 
 
+@pytest.mark.parametrize(
+    ("weights", "expected"),
+    (
+        (
+            [0.25, 0.05, 0.15, 0.25, 0.15, 0.1, 0.05],
+            [1.554595, 2.463784, 3.000000, 3.518378],
+        ),
+        (
+            [0.05, 0.05, 0.1, 0.15, 0.15, 0.25, 0.25],
+            [2.840000, 3.632973, 4.076216, 4.523243],
+        ),
+    ),
+)
+def test_weighted_quantile_no_nan(weights, expected):
+    # Expected values were calculated by running the reference implementation
+    # proposed in https://aakinshin.net/posts/weighted-quantiles/
+
+    da = DataArray([1, 1.9, 2.2, 3, 3.7, 4.1, 5])
+    q = [0.2, 0.4, 0.6, 0.8]
+    weights = DataArray(weights)
+
+    expected = DataArray(expected, coords={"quantile": q})
+    result = da.weighted(weights).quantile(q)
+
+    assert_allclose(expected, result)
+
+
+def test_weighted_quantile_zero_weights():
+
+    da = DataArray([0, 1, 2, 3])
+    weights = DataArray([1, 0, 1, 0])
+    q = 0.75
+
+    result = da.weighted(weights).quantile(q)
+    expected = DataArray([0, 2]).quantile(0.75)
+
+    assert_allclose(expected, result)
+
+
+def test_weighted_quantile_simple():
+    # Check that weighted quantiles return the same value as numpy quantiles
+    da = DataArray([0, 1, 2, 3])
+    w = DataArray([1, 0, 1, 0])
+
+    w_eps = DataArray([1, 0.0001, 1, 0.0001])
+    q = 0.75
+
+    expected = DataArray(np.quantile([0, 2], q), coords={"quantile": q})  # 1.5
+
+    assert_equal(expected, da.weighted(w).quantile(q))
+    assert_allclose(expected, da.weighted(w_eps).quantile(q), rtol=0.001)
+
+
+@pytest.mark.parametrize("skipna", (True, False))
+def test_weighted_quantile_nan(skipna):
+    # Check skipna behavior
+    da = DataArray([0, 1, 2, 3, np.nan])
+    w = DataArray([1, 0, 1, 0, 1])
+    q = [0.5, 0.75]
+
+    result = da.weighted(w).quantile(q, skipna=skipna)
+
+    if skipna:
+        expected = DataArray(np.quantile([0, 2], q), coords={"quantile": q})
+    else:
+        expected = DataArray(np.full(len(q), np.nan), coords={"quantile": q})
+
+    assert_allclose(expected, result)
+
+
+@pytest.mark.parametrize(
+    "da",
+    (
+        [1, 1.9, 2.2, 3, 3.7, 4.1, 5],
+        [1, 1.9, 2.2, 3, 3.7, 4.1, np.nan],
+        [np.nan, np.nan, np.nan],
+    ),
+)
+@pytest.mark.parametrize("q", (0.5, (0.2, 0.8)))
+@pytest.mark.parametrize("skipna", (True, False))
+@pytest.mark.parametrize("factor", [1, 3.14])
+def test_weighted_quantile_equal_weights(da, q, skipna, factor):
+    # if all weights are equal (!= 0), should yield the same result as quantile
+
+    da = DataArray(da)
+    weights = xr.full_like(da, factor)
+
+    expected = da.quantile(q, skipna=skipna)
+    result = da.weighted(weights).quantile(q, skipna=skipna)
+
+    assert_allclose(expected, result)
+
+
+@pytest.mark.skip(reason="`method` argument is not currently exposed")
+@pytest.mark.parametrize(
+    "da",
+    (
+        [1, 1.9, 2.2, 3, 3.7, 4.1, 5],
+        [1, 1.9, 2.2, 3, 3.7, 4.1, np.nan],
+        [np.nan, np.nan, np.nan],
+    ),
+)
+@pytest.mark.parametrize("q", (0.5, (0.2, 0.8)))
+@pytest.mark.parametrize("skipna", (True, False))
+@pytest.mark.parametrize(
+    "method",
+    [
+        "linear",
+        "interpolated_inverted_cdf",
+        "hazen",
+        "weibull",
+        "median_unbiased",
+        "normal_unbiased2",
+    ],
+)
+def test_weighted_quantile_equal_weights_all_methods(da, q, skipna, factor, method):
+    # If all weights are equal (!= 0), should yield the same result as numpy quantile
+
+    da = DataArray(da)
+    weights = xr.full_like(da, 3.14)
+
+    expected = da.quantile(q, skipna=skipna, method=method)
+    result = da.weighted(weights).quantile(q, skipna=skipna, method=method)
+
+    assert_allclose(expected, result)
+
+
+def test_weighted_quantile_bool():
+    # https://github.com/pydata/xarray/issues/4074
+    da = DataArray([1, 1])
+    weights = DataArray([True, True])
+    q = 0.5
+
+    expected = DataArray([1], coords={"quantile": [q]}).squeeze()
+    result = da.weighted(weights).quantile(q)
+
+    assert_equal(expected, result)
+
+
+@pytest.mark.parametrize("q", (-1, 1.1, (0.5, 1.1), ((0.2, 0.4), (0.6, 0.8))))
+def test_weighted_quantile_with_invalid_q(q):
+
+    da = DataArray([1, 1.9, 2.2, 3, 3.7, 4.1, 5])
+    q = np.asarray(q)
+    weights = xr.ones_like(da)
+
+    if q.ndim <= 1:
+        with pytest.raises(ValueError, match="q values must be between 0 and 1"):
+            da.weighted(weights).quantile(q)
+    else:
+        with pytest.raises(ValueError, match="q must be a scalar or 1d"):
+            da.weighted(weights).quantile(q)
+
+
 @pytest.mark.parametrize(
     ("weights", "expected"), (([4, 6], 2.0), ([1, 0], np.nan), ([0, 0], np.nan))
 )
@@ -443,6 +599,7 @@ def check_weighted_operations(data, weights, dim, skipna):
 @pytest.mark.parametrize("dim", ("a", "b", "c", ("a", "b"), ("a", "b", "c"), None))
 @pytest.mark.parametrize("add_nans", (True, False))
 @pytest.mark.parametrize("skipna", (None, True, False))
+@pytest.mark.filterwarnings("ignore:invalid value encountered in sqrt")
 def test_weighted_operations_3D(dim, add_nans, skipna):
 
     dims = ("a", "b", "c")
@@ -465,21 +622,62 @@ def test_weighted_operations_3D(dim, add_nans, skipna):
     check_weighted_operations(data, weights, dim, skipna)
 
 
-def test_weighted_operations_nonequal_coords():
+@pytest.mark.parametrize("dim", ("a", "b", "c", ("a", "b"), ("a", "b", "c"), None))
+@pytest.mark.parametrize("q", (0.5, (0.1, 0.9), (0.2, 0.4, 0.6, 0.8)))
+@pytest.mark.parametrize("add_nans", (True, False))
+@pytest.mark.parametrize("skipna", (None, True, False))
+def test_weighted_quantile_3D(dim, q, add_nans, skipna):
 
+    dims = ("a", "b", "c")
+    coords = dict(a=[0, 1, 2], b=[0, 1, 2, 3], c=[0, 1, 2, 3, 4])
+
+    data = np.arange(60).reshape(3, 4, 5).astype(float)
+
+    # add approximately 25 % NaNs (https://stackoverflow.com/a/32182680/3010700)
+    if add_nans:
+        c = int(data.size * 0.25)
+        data.ravel()[np.random.choice(data.size, c, replace=False)] = np.NaN
+
+    da = DataArray(data, dims=dims, coords=coords)
+
+    # Weights are all ones, because we will compare against DataArray.quantile (non-weighted)
+    weights = xr.ones_like(da)
+
+    result = da.weighted(weights).quantile(q, dim=dim, skipna=skipna)
+    expected = da.quantile(q, dim=dim, skipna=skipna)
+
+    assert_allclose(expected, result)
+
+    ds = da.to_dataset(name="data")
+    result2 = ds.weighted(weights).quantile(q, dim=dim, skipna=skipna)
+
+    assert_allclose(expected, result2.data)
+
+
+def test_weighted_operations_nonequal_coords():
+    # There are no weights for a == 4, so that data point is ignored.
     weights = DataArray(np.random.randn(4), dims=("a",), coords=dict(a=[0, 1, 2, 3]))
     data = DataArray(np.random.randn(4), dims=("a",), coords=dict(a=[1, 2, 3, 4]))
-
     check_weighted_operations(data, weights, dim="a", skipna=None)
 
+    q = 0.5
+    result = data.weighted(weights).quantile(q, dim="a")
+    # Expected value computed using code from https://aakinshin.net/posts/weighted-quantiles/ with values at a=1,2,3
+    expected = DataArray([0.9308707], coords={"quantile": [q]}).squeeze()
+    assert_allclose(result, expected)
+
     data = data.to_dataset(name="data")
     check_weighted_operations(data, weights, dim="a", skipna=None)
 
+    result = data.weighted(weights).quantile(q, dim="a")
+    assert_allclose(result, expected.to_dataset(name="data"))
+
 
 @pytest.mark.parametrize("shape_data", ((4,), (4, 4), (4, 4, 4)))
 @pytest.mark.parametrize("shape_weights", ((4,), (4, 4), (4, 4, 4)))
 @pytest.mark.parametrize("add_nans", (True, False))
 @pytest.mark.parametrize("skipna", (None, True, False))
+@pytest.mark.filterwarnings("ignore:invalid value encountered in sqrt")
 def test_weighted_operations_different_shapes(
     shape_data, shape_weights, add_nans, skipna
 ):
@@ -504,7 +702,8 @@ def test_weighted_operations_different_shapes(
 
 
 @pytest.mark.parametrize(
-    "operation", ("sum_of_weights", "sum", "mean", "sum_of_squares", "var", "std")
+    "operation",
+    ("sum_of_weights", "sum", "mean", "sum_of_squares", "var", "std", "quantile"),
 )
 @pytest.mark.parametrize("as_dataset", (True, False))
 @pytest.mark.parametrize("keep_attrs", (True, False, None))
@@ -518,22 +717,23 @@ def test_weighted_operations_keep_attr(operation, as_dataset, keep_attrs):
 
     data.attrs = dict(attr="weights")
 
-    result = getattr(data.weighted(weights), operation)(keep_attrs=True)
+    kwargs = {"keep_attrs": keep_attrs}
+    if operation == "quantile":
+        kwargs["q"] = 0.5
+
+    result = getattr(data.weighted(weights), operation)(**kwargs)
 
     if operation == "sum_of_weights":
-        assert weights.attrs == result.attrs
+        assert result.attrs == (weights.attrs if keep_attrs else {})
+        assert result.attrs == (weights.attrs if keep_attrs else {})
     else:
-        assert data.attrs == result.attrs
-
-    result = getattr(data.weighted(weights), operation)(keep_attrs=None)
-    assert not result.attrs
-
-    result = getattr(data.weighted(weights), operation)(keep_attrs=False)
-    assert not result.attrs
+        assert result.attrs == (weights.attrs if keep_attrs else {})
+        assert result.attrs == (data.attrs if keep_attrs else {})
 
 
 @pytest.mark.parametrize(
-    "operation", ("sum_of_weights", "sum", "mean", "sum_of_squares", "var", "std")
+    "operation",
+    ("sum_of_weights", "sum", "mean", "sum_of_squares", "var", "std", "quantile"),
 )
 def test_weighted_operations_keep_attr_da_in_ds(operation):
     # GH #3595
@@ -542,22 +742,31 @@ def test_weighted_operations_keep_attr_da_in_ds(operation):
     data = DataArray(np.random.randn(2, 2), attrs=dict(attr="data"))
     data = data.to_dataset(name="a")
 
-    result = getattr(data.weighted(weights), operation)(keep_attrs=True)
+    kwargs = {"keep_attrs": True}
+    if operation == "quantile":
+        kwargs["q"] = 0.5
+
+    result = getattr(data.weighted(weights), operation)(**kwargs)
 
     assert data.a.attrs == result.a.attrs
 
 
+@pytest.mark.parametrize("operation", ("sum_of_weights", "sum", "mean", "quantile"))
 @pytest.mark.parametrize("as_dataset", (True, False))
-def test_weighted_bad_dim(as_dataset):
+def test_weighted_bad_dim(operation, as_dataset):
 
     data = DataArray(np.random.randn(2, 2))
     weights = xr.ones_like(data)
     if as_dataset:
         data = data.to_dataset(name="data")
 
+    kwargs = {"dim": "bad_dim"}
+    if operation == "quantile":
+        kwargs["q"] = 0.5
+
     error_msg = (
         f"{data.__class__.__name__}Weighted"
         " does not contain the dimensions: {'bad_dim'}"
     )
     with pytest.raises(ValueError, match=error_msg):
-        data.weighted(weights).mean("bad_dim")
+        getattr(data.weighted(weights), operation)(**kwargs)
